#include "datastructure.h"
#include <stdio.h>
#include <stdlib.h>

#include <time.h>

const char *delimiter = ", ";

elementNodePtr createValueElement(struct Value _value){
	elementNodePtr retval = (elementNodePtr) malloc(sizeof(struct elementNode));

	retval->type = value;
	retval->u._value = _value;

	return retval;
}

elementNodePtr createStatementElement(statementNodePtr stmt){
	elementNodePtr retval = (elementNodePtr) malloc(sizeof(struct elementNode));

	retval->type = statement;
	retval->u.stmt = stmt;

	return retval;
}
//*************************************************

statementNodePtr createStatement(enumerationNodePtr ePtr){

	statementNodePtr retval = (statementNodePtr) malloc(sizeof(struct statementNode));

	retval->ePtr = ePtr;

	return retval;
}
//*************************************************
enumerationNodePtr createEnumerationNode(enumerationNodePtr next, elementNodePtr element){

	enumerationNodePtr retval = (enumerationNodePtr) malloc(sizeof(struct enumerationNode));

	retval->next = next;
	retval->element = element;

	return retval;
}

//************************************************
// Methods for traversing and processing the tree

char* processStatement(statementNodePtr root){
	char *str = processEnum(root->ePtr);
	return str;
}

char* processEnumElement(elementNodePtr element){
	if (element == NULL) {
		// ignore empty elements
		return NULL;
	}

	switch(element->type){

	case value: 

		switch (element->u._value.type) {

		case wordType: //printf("type word %s\n", element->u._value.u.svalue );
			return element->u._value.u.svalue;
		break;	

		case numberType: //printf("type number %d\n", element->u._value.u.ivalue );
			return NULL; // will not output numbers at all
		break;	

		default:
		break;
		}

	break; 

	case statement:
		//printf("type statement\n" );
		// recursion again
		return processStatement(element->u.stmt);
	break; 

	default:
		printf("unexpected type, exiting\n" );
		exit (0);
	}
}

char* processEnum(enumerationNodePtr list){
	// we are handling here a linked list
	if (list != NULL){
		
		if (list->next != NULL) {
			// recursion, we are going to the end of the list first
			char *strList = processEnum(list->next); // string represent values in the list
			int lenStrList = strlen(strList);
			
			char *strElement = processEnumElement(list->element); // string represents values in the last element
;
			if (strElement) {

				int lenStrElement = strlen(strElement);

				int delimeterLen = strlen(delimiter);

				int r = rand() % 2;

				if (r == 1) {
			    		// Forming a line as follows: strList + delimiter + strElement
			    		strList = (char*) realloc (strList, sizeof(char)* (lenStrList + lenStrElement + delimeterLen));
			    		strcpy (strList + lenStrList , delimiter);
			    		strcpy (strList + lenStrList + strlen(delimiter), strElement);
			    		return strList;
				} else {
			    		// Forming a line as follows: strElement + delimiter + strList
					// can't use strElement as destination, it will spoil the value
					
					char* strTemp = (char*) malloc (sizeof(char)* (lenStrList + lenStrElement + delimeterLen));
					strcpy (strTemp, strElement);
					strcpy (strTemp + lenStrElement, delimiter);
					strcpy (strTemp + lenStrElement + strlen(delimiter), strList);
					return strTemp;
				}
			} else {
			    return strList;
			}

		} else {
			// we get here if the element of enumeration isn't composite 
			char *str =  processEnumElement(list->element);
			return str;
		}
	}
}

