%{
#include <stdio.h>
#include <string.h>

#include "datastructure.h"

//int yydebug = 0;

extern int current_line_number;
extern int current_pos;

statementNodePtr root = NULL;

void yyerror(const char *str)
{
	fprintf(stderr,"error: [%s] in line [%d] position [%d]\n",str, current_line_number, current_pos);
}

int yywrap()
{
	return 1;
}

int main(int argc, char *argv[])
{
	/* this block makes binary read from the file instead of stdin */
	if (argc < 2) {
		printf("usage: %s <input file>\n", argv[0]);
	}

	extern FILE *yyin;
	yyin = fopen( argv[1], "r" );

	// now parsing the file
	yyparse();
     
	// Intializes random number generator
	srand(time(NULL));

        // processing the tree
	if (root) {
	    char *str = processStatement(root);
	    // we get shuffled values here, numbers are excluded
	    printf("%s\n", str);
	}

	/* Add to clean up memory leaks */
	yylex_destroy();

	return 0;
}

%}

%union {

	struct Value GenericValue; 

	// declaring types for composite components:
	statementNodePtr stmtType; 
	enumerationNodePtr enumerationType;
	elementNodePtr elType;

};

%token <GenericValue> WORD 
%token <GenericValue> NUMBER

%token OBRACE 
%token EBRACE 
%token SEMICOLON

%type <stmtType> statement
%type <enumerationType> enumeration
%type <elType> element

%%

statement:
	OBRACE enumeration EBRACE
	{
		$$ = createStatement($2);
		// we want to have in root exactly the root, not a pointer to some nested block 
		root = $$;

	}
	;

enumeration:
	{
		//"empty enumeration"
		$$ = createEnumerationNode(NULL, NULL);
        }
	|
	element
	{
		$$ = createEnumerationNode(NULL, $1);
	}
	|
	enumeration SEMICOLON element
	{
		$$ = createEnumerationNode($1, $3);
	}
	;

element:
	WORD
	{
		$$ = createValueElement($1);
	}
	|
	NUMBER
	{
		$$ = createValueElement($1);
	}
	|
	statement
	{
		$$ = createStatementElement($1);
	}


